@@ -21,6 +21,21 @@ else:
         print(a)
 
 
+def func(_):
+    for _ in range(1):
+        ...
+
+
+def func(foo):
+    for _ in bar(foo for foo in [1]):
+        pass
+
+
+def func(foo):
+    for _ in bar((foo := 1) for foo in [1]):
+        pass
+
+
 # Errors
 def func(a):
     for a in range(1):
