@@ -1,8 +1,8 @@
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::ExprCall;
 use ruff_text_size::Ranged;
 
+use crate::Violation;
 use crate::checkers::ast::Checker;
 
 /// ## What it does
@@ -25,27 +25,26 @@ use crate::checkers::ast::Checker;
 ///
 /// ssl.wrap_socket(ssl_version=ssl.PROTOCOL_TLSv1_2)
 /// ```
-#[violation]
-pub struct SslWithNoVersion;
+#[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.2.0")]
+pub(crate) struct SslWithNoVersion;
 
 impl Violation for SslWithNoVersion {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`ssl.wrap_socket` called without an `ssl_version``")
+        "`ssl.wrap_socket` called without an `ssl_version``".to_string()
     }
 }
 
 /// S504
-pub(crate) fn ssl_with_no_version(checker: &mut Checker, call: &ExprCall) {
+pub(crate) fn ssl_with_no_version(checker: &Checker, call: &ExprCall) {
     if checker
         .semantic()
         .resolve_qualified_name(call.func.as_ref())
         .is_some_and(|qualified_name| matches!(qualified_name.segments(), ["ssl", "wrap_socket"]))
     {
         if call.arguments.find_keyword("ssl_version").is_none() {
-            checker
-                .diagnostics
-                .push(Diagnostic::new(SslWithNoVersion, call.range()));
+            checker.report_diagnostic(SslWithNoVersion, call.range());
         }
     }
 }
