@@ -1,21 +1,22 @@
-use ruff_diagnostics::{AlwaysFixableViolation, Diagnostic, Edit, Fix};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast as ast;
+use ruff_python_ast::ExprGenerator;
 use ruff_python_ast::comparable::ComparableExpr;
 use ruff_python_ast::parenthesize::parenthesized_range;
-use ruff_python_ast::ExprGenerator;
-use ruff_text_size::{Ranged, TextSize};
+use ruff_python_parser::TokenKind;
+use ruff_text_size::{Ranged, TextRange, TextSize};
 
 use crate::checkers::ast::Checker;
+use crate::{AlwaysFixableViolation, Edit, Fix};
 
-use super::helpers;
+use crate::rules::flake8_comprehensions::helpers;
 
 /// ## What it does
-/// Checks for unnecessary generators that can be rewritten as `list`
-/// comprehensions (or with `list` directly).
+/// Checks for unnecessary generators that can be rewritten as list
+/// comprehensions (or with `list()` directly).
 ///
 /// ## Why is this bad?
-/// It is unnecessary to use `list` around a generator expression, since
+/// It is unnecessary to use `list()` around a generator expression, since
 /// there are equivalent comprehensions for these types. Using a
 /// comprehension is clearer and more idiomatic.
 ///
@@ -23,7 +24,7 @@ use super::helpers;
 /// `list(x for x in foo)`, it's better to use `list(foo)` directly, since it's
 /// even more direct.
 ///
-/// ## Examples
+/// ## Example
 /// ```python
 /// list(f(x) for x in foo)
 /// list(x for x in foo)
@@ -40,8 +41,9 @@ use super::helpers;
 /// ## Fix safety
 /// This rule's fix is marked as unsafe, as it may occasionally drop comments
 /// when rewriting the call. In most cases, though, comments will be preserved.
-#[violation]
-pub struct UnnecessaryGeneratorList {
+#[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.61")]
+pub(crate) struct UnnecessaryGeneratorList {
     short_circuit: bool,
 }
 
@@ -49,9 +51,9 @@ impl AlwaysFixableViolation for UnnecessaryGeneratorList {
     #[derive_message_formats]
     fn message(&self) -> String {
         if self.short_circuit {
-            format!("Unnecessary generator (rewrite using `list()`)")
+            "Unnecessary generator (rewrite using `list()`)".to_string()
         } else {
-            format!("Unnecessary generator (rewrite as a `list` comprehension)")
+            "Unnecessary generator (rewrite as a list comprehension)".to_string()
         }
     }
 
@@ -59,13 +61,13 @@ impl AlwaysFixableViolation for UnnecessaryGeneratorList {
         if self.short_circuit {
             "Rewrite using `list()`".to_string()
         } else {
-            "Rewrite as a `list` comprehension".to_string()
+            "Rewrite as a list comprehension".to_string()
         }
     }
 }
 
 /// C400 (`list(generator)`)
-pub(crate) fn unnecessary_generator_list(checker: &mut Checker, call: &ast::ExprCall) {
+pub(crate) fn unnecessary_generator_list(checker: &Checker, call: &ast::ExprCall) {
     let Some(argument) = helpers::exactly_one_argument_with_matching_function(
         "list",
         &call.func,
@@ -74,49 +76,48 @@ pub(crate) fn unnecessary_generator_list(checker: &mut Checker, call: &ast::Expr
     ) else {
         return;
     };
-    if !checker.semantic().has_builtin_binding("list") {
-        return;
-    }
 
-    let Some(ExprGenerator {
+    let ast::Expr::Generator(ExprGenerator {
         elt,
         generators,
         parenthesized,
         ..
-    }) = argument.as_generator_expr()
+    }) = argument
     else {
         return;
     };
 
+    if !checker.semantic().has_builtin_binding("list") {
+        return;
+    }
+
     // Short-circuit: given `list(x for x in y)`, generate `list(y)` (in lieu of `[x for x in y]`).
     if let [generator] = generators.as_slice() {
         if generator.ifs.is_empty() && !generator.is_async {
             if ComparableExpr::from(elt) == ComparableExpr::from(&generator.target) {
-                let mut diagnostic = Diagnostic::new(
-                    UnnecessaryGeneratorList {
-                        short_circuit: true,
-                    },
-                    call.range(),
-                );
                 let iterator = format!("list({})", checker.locator().slice(generator.iter.range()));
-                diagnostic.set_fix(Fix::unsafe_edit(Edit::range_replacement(
-                    iterator,
-                    call.range(),
-                )));
-                checker.diagnostics.push(diagnostic);
+                let fix = Fix::unsafe_edit(Edit::range_replacement(iterator, call.range()));
+                checker
+                    .report_diagnostic(
+                        UnnecessaryGeneratorList {
+                            short_circuit: true,
+                        },
+                        call.range(),
+                    )
+                    .set_fix(fix);
                 return;
             }
         }
     }
 
     // Convert `list(f(x) for x in y)` to `[f(x) for x in y]`.
-    let mut diagnostic = Diagnostic::new(
+    let mut diagnostic = checker.report_diagnostic(
         UnnecessaryGeneratorList {
             short_circuit: false,
         },
         call.range(),
     );
-    diagnostic.set_fix({
+    let fix = {
         // Replace `list(` with `[`.
         let call_start = Edit::replacement(
             "[".to_string(),
@@ -125,11 +126,16 @@ pub(crate) fn unnecessary_generator_list(checker: &mut Checker, call: &ast::Expr
         );
 
         // Replace `)` with `]`.
-        let call_end = Edit::replacement(
-            "]".to_string(),
-            call.arguments.end() - TextSize::from(1),
-            call.end(),
-        );
+        // Place `]` at argument's end or at trailing comma if present
+        let after_arg_tokens = checker
+            .tokens()
+            .in_range(TextRange::new(argument.end(), call.end()));
+        let right_bracket_loc = after_arg_tokens
+            .iter()
+            .find(|token| token.kind() == TokenKind::Comma)
+            .map_or(call.arguments.end(), Ranged::end)
+            - TextSize::from(1);
+        let call_end = Edit::replacement("]".to_string(), right_bracket_loc, call.end());
 
         // Remove the inner parentheses, if the expression is a generator. The easiest way to do
         // this reliably is to use the printer.
@@ -139,7 +145,7 @@ pub(crate) fn unnecessary_generator_list(checker: &mut Checker, call: &ast::Expr
             let range = parenthesized_range(
                 argument.into(),
                 (&call.arguments).into(),
-                checker.indexer().comment_ranges(),
+                checker.comment_ranges(),
                 checker.locator().contents(),
             )
             .unwrap_or(argument.range());
@@ -153,7 +159,6 @@ pub(crate) fn unnecessary_generator_list(checker: &mut Checker, call: &ast::Expr
         } else {
             Fix::unsafe_edits(call_start, [call_end])
         }
-    });
-
-    checker.diagnostics.push(diagnostic);
+    };
+    diagnostic.set_fix(fix);
 }
